@@ -13,9 +13,12 @@ use DBD::SQLite;
 use DBIx::Migration;
 use File::ShareDir ':ALL';
 use Text::VimColor;
+use Image::WordCloud;
 use strict;
 use warnings;
 
+our $VERSION = "0.3";
+
 has 'dbh' => (is=>'rw');
 
 has 'dbfile' => (is => 'rw');
@@ -139,17 +142,22 @@ sub addRepo
 sub getRepos
 {
   my $self = shift;
+  my $order = shift || "name";
+  my $dir   = shift || "DESC";
   my @repos;
 
   my $sth = $self->dbh()->prepare("SELECT repository.reponame as name, repository.dir as dir,
                                           repository.description as description,
-                                          repository.lastchange as lastchange from repository order by name");
+                                          repository.lastchange as lastchange from repository order by " . $order . " ". $dir );
   $sth->execute();
 
   my $tz=DateTime::TimeZone->new( name => 'local' );
+  my $count=0;
 
   while (my $repo = $sth->fetchrow_hashref)
   {
+    $count++;
+    $repo->{count}=$count;
     $repo->{lastchange} = DateTime->from_epoch( epoch => $repo->{lastchange}, time_zone=>$tz)->ymd();
     my $sth2 = $self->dbh()->prepare("SELECT tag from repository_tags where reponame = ?");
     $sth2->execute($repo->{name});
@@ -182,7 +190,35 @@ sub getRepos
   return \@repos;
 }
 
+sub getAllTags
+{
+  my $self = shift;
+  my @tags;
 
+  my $sth = $self->dbh()->prepare("SELECT tag from repository_tags");
+  $sth->execute();
+  while (my $row = $sth->fetchrow_hashref)
+  {
+    push(@tags, $row->{tag});
+  }
+
+  return @tags;
+}
+
+sub getAllLangs
+{
+  my $self = shift;
+  my @langs;
+
+  my $sth = $self->dbh()->prepare("SELECT language from repository_languages");
+  $sth->execute();
+  while (my $row = $sth->fetchrow_hashref)
+  {
+    push(@langs, $row->{language});
+  }
+
+  return @langs;
+}
 
 sub generate {
   my( $self , $site ) = @_;
@@ -302,6 +338,35 @@ sub generate {
   my $dbrepos = $self->getRepos();
   my $nrrepos = @{$dbrepos};
 
+  #create word cloud image for all tags
+  my $font_dir = "/usr/share/fonts/truetype/freefont/";
+  my $wc_tags = Image::WordCloud->new(font_path => $font_dir, background=>[255,255,255], font=>"FreeSans.ttf", image_size=>[300,300] );
+  $wc_tags->words(join ( ' ', $self->getAllTags ));
+  my $gd_tags = $wc_tags->cloud();
+  #make image background transparent
+  my $white = $gd_tags->colorAllocate(255,255,255);
+  $gd_tags->transparent($white);
+
+  readpipe("mkdir -p " . $self->destination() . "/images/");
+  open(my $fh, '>', $self->destination()."/images/wc_tags.png");
+  binmode $fh;
+  print $fh $gd_tags->png();
+  close($fh);
+
+  #create word cloud image for all languages
+  my $wc_langs = Image::WordCloud->new(font_path => $font_dir, background=>[255,255,255], font=>"FreeSans.ttf", image_size=>[300,300] );
+  $wc_langs->words(join ( ' ', $self->getAllLangs ));
+  my $gd_langs = $wc_langs->cloud();
+  #make image background transparent
+  $white = $gd_langs->colorAllocate(255,255,255);
+  $gd_langs->transparent($white);
+
+  open($fh, '>', $self->destination()."/images/wc_langs.png");
+  binmode $fh;
+  print $fh $gd_langs->png();
+  close($fh);
+
+
   # create the index.htm page
   my $index = HiD::Page->new({
     dest_dir       => $self->destination(),
@@ -310,7 +375,13 @@ sub generate {
     input_filename => $self->index_layout(),
     layouts        => $site->layouts,
   });
+  my $last_repos = $self->getRepos("lastchange", "DESC");
+  splice @{$last_repos}, 6;
+
+  $index->metadata->{repos} = $last_repos;
   $index->metadata->{nrrepos} = $nrrepos;
+  $index->metadata->{tagcloud}="/images/wc_tags.png";
+  $index->metadata->{langcloud}="/images/wc_langs.png";
   $site->add_input( "INDEX" => 'page' );
   $site->add_object( $index );
   $site->INFO( "* Injected Index Page");
@@ -480,7 +551,7 @@ HiD::Generator::GitRepositories - HiD GitRepository listing generator
 
 =head1 VERSION
 
-version 0.2
+version 0.3
 
 =head1 ATTRIBUTES
 
@@ -550,6 +621,14 @@ add the given Repo and the information to the database
 
 fetch all repos from database and return as an array from hashrefs
 
+=head2 getAllTags
+
+fetch all tags from database and return as an array
+
+=head2 getAllLangs
+
+fetch all languages from database and return as an array
+
 =head1 AUTHOR
 
 Dominik Meyer <dmeyer@federationhq.de>
